/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.orchestration;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.message.ExchangeImpl;
import org.ow2.petals.component.framework.process.async.AsyncContext;
import org.ow2.petals.orchestration.core.ExchangeContextImpl;
import org.ow2.petals.orchestration.core.OrchestrationAsyncContext;
import org.ow2.petals.orchestration.core.OrchestratorImpl;

public abstract class PojoScript {
	
	Logger logger_;
	AbstractJBIListener jbiListener_;
	ComponentContext ctx_;
	String installRoot_;
	
	/**
	 * Setter method called by the SE-POJO, to inject the JBI listener
	 * @deprecated (inadequately used to exclude from javadoc...)
	 */
	public final void setJBIListener(AbstractJBIListener jbiListener) {
		this.jbiListener_ = jbiListener;
	}
	/**
	 * Setter method called by the SE-POJO, to inject the component context
	 * @deprecated (inadequately used to exclude from javadoc...)
	 */
	public final void setComponentContext(ComponentContext ctx) {
		this.ctx_ = ctx;
	}
	/**
	 * Setter method called by the SE-POJO, to inject the logger
	 * @deprecated (inadequately used to exclude from javadoc...)
	 */
	public final void setLogger(Logger logger) {
		logger_ = logger;
	}
	/**
     * Setter method called by the SE-POJO, to inject the SU install root
     * @deprecated (inadequately used to exclude from javadoc...)
     */
    public final void setInstallRoot(String installRoot) {
        installRoot_ = installRoot;
    }
    
	/**
	 * Called by the SE-POJO upon JBI exchange
	 * @param exchange
	 * @return
	 * @throws Exception
	 * @deprecated (inadequately used to exclude from javadoc...)
	 */
	public final boolean onExchange(Exchange exchange) throws Exception {
		if(exchange.isConsumerRole()) return false;
		else {
			onExchange(new ExchangeContextImpl(exchange, null), newOrchestrator(exchange));
			return true;
		}
	}
	
	/**
	 * Called by the SE-POJO upon asynchronous JBI exchange
	 * @param exchange
	 * @param asyncContext
	 * @return
	 * @throws Exception
	 * @deprecated (inadequately used to exclude from javadoc...)
	 */
	public final boolean onAsyncExchange(Exchange exchange, AsyncContext asyncContext) throws Exception {
		//System.out.println("active=" + exchange.isActiveStatus() + " done=" + exchange.isDoneStatus() + " error=" + exchange.isErrorStatus());
		OrchestrationAsyncContext ctx = (OrchestrationAsyncContext)asyncContext;
		OrchestratorImpl original = (OrchestratorImpl)ctx.getOriginalOrchestrator();
		// Process only OUT messages (= answers to in-out requests),
		// not acks or anything related to protocol concerns...
		// When so, a "return false" (= "I manage protocol concerns myself" ?) works fine...
		// but returning true causes exceptions in Petals (sic). Why, I'd like to understand !
		if(original != null && !exchange.isInMessage()) {
            // TODO where is it reenabled?!?!
			((ExchangeImpl) exchange).setCheckRole(false);
			OrchestratorImpl result = newOrchestrator(exchange);
			original.setCurrent(true, result);
			onAsyncExchange(new ExchangeContextImpl(exchange, original), result);
			return true;
		} else {
			return false;
		}
	}
	
	private OrchestratorImpl newOrchestrator(Exchange exchange) throws Exception {
        return new OrchestratorImpl(exchange, jbiListener_, ctx_, installRoot_);
	}
	
	/**
	 * Get the current logger (default logger: org.ow2.petals.orchestration, Level.ALL).
	 * @return The current logger.
	 */
	public Logger getLogger() {
		if(logger_ == null) {
			Logger logger = Logger.getLogger("org.ow2.petals.orchestration");
			logger.setLevel(Level.ALL);
			return logger;
		}
		else return logger_;
	}

	/**
	 * Initialize method: override if needed (default implementation does nothing).
	 */
	public void init() {
		return; // default implementation
	}
	
	/**
	 * Method called on JBI exchange (to be implemented, in order to manage the exchange).
	 * @param ectx The current exchange context.
	 * @param o An orchestrator, initialized for the current exchange.
	 * @throws Exception
	 */
	public abstract void onExchange(ExchangeContext ectx, Orchestrator o) throws Exception;
	
	/**
	 * Method called on asynchronous JBI exchange (to be implemented if async support is needed).
	 * Default implementation does nothing.
	 * @param ectx The current exchange context.
	 * @param current An orchestrator, initialized for the current exchange.
	 * @throws Exception
	 */
	public void onAsyncExchange(ExchangeContext ectx, Orchestrator current)
	throws Exception {
		return; // default implementation
	}

}
